<?php

namespace Payshop\Core\Action;

use \Magento\Sales\Model\OrderRepository;
use \Magento\Sales\Api\Data\OrderInterface;
use Payshop\Core\Validation\ConfirmCardPayment as ValidationConfirmCardPayment;

class ConfirmCardPayment
{
    /**
     * @var OrderRepository
     */
    private OrderRepository $orderRepository;

    /**
     * @var CreatePayment
     */
    private CreatePayment $createPayment;

    /**
     * ConfirmCardPayment constructor.
     * 
     * @param OrderRepository $orderRepository
     * @param CreatePayment $createPayment
     */
    public function __construct(
        OrderRepository $orderRepository,
        CreatePayment $createPayment
    ) {
        $this->orderRepository = $orderRepository;
        $this->createPayment = $createPayment;
    }

    /**
     * Confirm efective order value and create payment
     * 
     * @param int $orderId
     * @param float $efectiveOrderValue
     * @return void
     * @throws \Exception
     */
    public function execute(?int $orderId, string $efectiveOrderValue): void
    {
        $this->validateInput($orderId, $efectiveOrderValue);
        
        $order = $this->orderRepository->get($orderId);
        $this->validateOrder($order);
        $this->checkAmount($order, $efectiveOrderValue);

        $payment = $order->getPayment();

        $instrumentId = $payment->getAdditionalInformation('instrument_id');
        $paymentId = $this->createPayment->execute($instrumentId, $efectiveOrderValue);
        
        $this->saveOrderDetails($order, $efectiveOrderValue, $paymentId);
    }

    /**
     * Validate input data
     * 
     * @param int $orderId
     * @param string $efectiveOrderValue
     * @return void
     * @throws \Exception
     */
    private function validateInput(int $orderId, string $efectiveOrderValue): void
    {
        if (!$orderId) {
            throw new \Exception(__('Order ID is required'));
        }

        if (!$efectiveOrderValue) {
            throw new \Exception(__('Efective order value is required'));
        }

        if (!is_numeric($efectiveOrderValue)) {
            throw new \Exception(__('Efective order value must be a number'));
        }
    }

    /**
     * Validate order data
     * 
     * @param OrderInterface $order
     * @return void
     * 
     * @throws \Exception
     */
    private function validateOrder(OrderInterface $order): void
    {
        if (!$order) {
            throw new \Exception(__('Order not found'));
        }

        if (!ValidationConfirmCardPayment::isProcessingStatus($order)) {
            throw new \Exception(__('Order must be in processing payment state'));
        }

        if (!ValidationConfirmCardPayment::isCard($order)) {
            throw new \Exception(__('Order must be paid with Card'));
        }

        if (ValidationConfirmCardPayment::isPaymentConfirmed($order)) {
            throw new \Exception(__('Order is already confirmed'));
        }

        if (ValidationConfirmCardPayment::isDateLimitToConfirmPaymentExpired($order)) {
            throw new \Exception(__('Date limit to confirm payment expired'));
        }
    }

    /**
     * Check if efective order value is valid
     * 
     * @param OrderInterface $order
     * @param float $efectiveOrderValue
     * @return void
     * 
     * @throws \Exception
     */
    private function checkAmount(OrderInterface $order, float $efectiveOrderValue): void
    {
        $orderValue = $order->getGrandTotal();

        if ($efectiveOrderValue < 0) {
            throw new \Exception(__('Efective order value must be greater than zero'));
        }

        if ($efectiveOrderValue > $orderValue) {
            throw new \Exception(__('Efective order value must be less than or equal to order value'));
        }
    }

    /**
     * Save data confirmation in order
     * 
     * @param OrderInterface $order
     * @param float $efectiveOrderValue
     * @param string $paymentId
     * @return void
     */
    private function saveOrderDetails(OrderInterface $order, float $efectiveOrderValue, string $paymentId): void
    {
        $payment = $order->getPayment();

        $payment->setAdditionalInformation('efective_order_value', $efectiveOrderValue);
        $payment->setAdditionalInformation('confirmation_date', date('Y-m-d H:i:s'));
        $payment->setAdditionalInformation('payment_id', $paymentId);

        $order->setTotalPaid($efectiveOrderValue);

        $this->orderRepository->save($order);
    }
}