<?php

namespace Payshop\Core\Action;

use Payshop\Core\Helper\PaymentMethods;
use Payshop\Core\Gateway\SDK\PayshopSDKClientFactory;

class CreateInstrument
{
    /**
     * @var PaymentMethods
     */
    private PaymentMethods $paymentMethods;

    /**
     * @var PayshopSDKClientFactory
     */
    private PayshopSDKClientFactory $payshopSDKClientFactory;


    /**
     * CreateInstrument constructor.
     * 
     * @param PaymentMethods $paymentMethods
     * @param PayshopSDKClientFactory $payshopSDKClientFactory
     */
    public function __construct(
        PaymentMethods $paymentMethods,
        PayshopSDKClientFactory $payshopSDKClientFactory
    ) {
        $this->paymentMethods = $paymentMethods;
        $this->payshopSDKClientFactory = $payshopSDKClientFactory;
    }

    /**
     * Create instrument on Payshop
     *
     * @param array $instrumentData
     * @return array
     */
    public function execute(array $instrumentData): array
    {
        $payshopClient = $this->payshopSDKClientFactory->getInstance();

        $instrumentResponse = $payshopClient->createInstrument($instrumentData);
        $this->paymentMethods->checkResponse($instrumentResponse);

        return $instrumentResponse['response'];
    }

    /**
     * Get instrument request data
     * 
     * @param object $payment
     * @param array $data
     * @return array
     */
    public function getData(object $payment, array $data = []): array
    {
        $isPayshopReference = $payment->getMethod() === 'payshop_payshop_reference';
        $isMultibancoReference = $payment->getMethod() === 'payshop_multibanco_reference';

        if ($isPayshopReference || $isMultibancoReference) {
            return [];
        }

        $isMBWay = $payment->getMethod() === 'payshop_mbway';

        if ($isMBWay) {
            return $this->getMBWayData($payment);
        }

        return $this->getCardData($data);
    }

    /**
     * Get the mbway data
     * 
     * @param object $payment
     * @return array
     */
    private function getMBWayData(object $payment): array
    {
        $phone = $payment->getAdditionalInformation('phone');

        return [
            'phone' => str_replace(' ', '', $phone)
        ];
    }

    /**
     * Get the card data
     * 
     * @param array $data
     * @return array
     */
    private function getCardData(array $data): array
    {
        if ($data['payment_in_two_steps']) {
            $data['capture_on_creation'] = false;
            $data['instrument_params'] = [
                'capture_on_creation' => false,
            ];
        }

        unset($data['payment_in_two_steps']);

        return $data;
    }
}
