<?php

namespace Payshop\Core\Controller\Card;

use Magento\Quote\Model\Quote;
use Payshop\Core\Validation\Card;
use Magento\Checkout\Model\Session;
use Payshop\Core\Action\CreateCharge;
use Payshop\Core\Action\CreateInstrument;
use Magento\Framework\App\RequestInterface;
use Magento\Payment\Gateway\ConfigInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Payment\Gateway\ErrorMapper\ErrorMessageMapper;

class Initialize implements HttpPostActionInterface
{
    /**
     * @var Session
     */
    private Session $session;

    /**
     * @var RequestInterface
     */
    private RequestInterface $request;
    
    /**
     * @var CreateCharge
     */
    protected CreateCharge $createCharge;
    
    /**
     * @var ConfigInterface
     */
    private ConfigInterface $cardConfig;

    /**
     * @var JsonFactory
     */
    protected JsonFactory $resultJsonFactory;
    
    /**
     * @var ErrorMessageMapper
     */
    private ErrorMessageMapper $errorMapping; 

    /**
     * @var CreateInstrument
     */
    protected CreateInstrument $createInstrument;

    /**
     * Initialize constructor.
     * 
     * @param Session $session
     * @param RequestInterface $request
     * @param CreateCharge $createCharge
     * @param ConfigInterface $cardConfig
     * @param JsonFactory $resultJsonFactory
     * @param ErrorMessageMapper $errorMapping
     * @param CreateInstrument $createInstrument
     */
    public function __construct(
        Session $session,
        RequestInterface $request,
        CreateCharge $createCharge,
        ConfigInterface $cardConfig,
        JsonFactory $resultJsonFactory,
        ErrorMessageMapper $errorMapping,
        CreateInstrument $createInstrument
    ) {
        $this->session = $session;
        $this->request = $request;
        $this->createCharge = $createCharge;
        $this->cardConfig = $cardConfig;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->errorMapping = $errorMapping;
        $this->createInstrument = $createInstrument;
    }

    /**
     * Create charge and instrument to pay with card
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultJsonFactory->create();
        
        try {
            $data = $this->request->getParams();
            Card::validate($data);

            $quote = $this->session->getQuote();

            $data['charge'] = $this->createCharge($quote);

            $instrument = $this->createInstrument($quote, $data);

            return $response->setData(['url' => $instrument['redirect']['url']]);
        } catch (\Throwable $th) {
            $messageCode = $th->getMessage();
            $message = $this->errorMapping->getMessage($messageCode)->getText();

            return $response
                ->setHttpResponseCode(400)
                ->setData(['message' => $message]);
        }
    }

    /**
     * Create charge on Payshop
     * 
     * @param Quote $quote
     * @return string
     */
    private function createCharge(Quote $quote): string
    {
        $chageRequest = $this->createCharge->getData($quote, 'payshop_card');

        $this->checkCurrency($chageRequest['currency']);

        $charge = $this->createCharge->execute($chageRequest);

        return $charge['id'];
    }

    /**
     * Create instrument on Payshop
     * 
     * @param Quote $quote
     * @param array $data
     * @return array
     */
    private function createInstrument(Quote $quote, array $data): array
    {
        $paymentInTwoSteps = $this->cardConfig->getValue(
            'payment_in_two_steps', 
            $quote->getStoreId()
        );

        $data['payment_in_two_steps'] = $paymentInTwoSteps ?? false;

        $instrumentRequest = $this->createInstrument->getData($quote->getPayment(), $data);

        $instrument = $this->createInstrument->execute($instrumentRequest);
        $this->checkInstrument($instrument);

        return $instrument;
    }

    /**
     * Check instrument
     * 
     * @param array $instrument
     * @return void
     * @throws \Exception
     */
    private function checkInstrument(array $instrument): void
    {
        $isNotPending = $instrument['status'] !== 'pending';
        $notHasRedirectUrl = !isset($instrument['redirect']['url']);

        if ($isNotPending || $notHasRedirectUrl) {
            throw new \Exception('400002');
        }
    }

    /**
     * Check currency
     * 
     * @param string $currency
     * @return void
     * @throws \Exception
     */
    private function checkCurrency(string $currency): void
    {
        if ($currency != 'EUR') {
            throw new \Exception('400003');
        }
    }

}