<?php

namespace Payshop\Core\Model\Events;

use Magento\Sales\Model\Order;
use Magento\Sales\Model\Spi\OrderResourceInterface;
use Payshop\Core\Gateway\SDK\PayshopSDKClientFactory;
use Magento\Sales\Api\Data\TransactionSearchResultInterfaceFactory;

class ProcessWebhookEvent
{
    /**
     * @var OrderResourceInterface
     */
    private OrderResourceInterface $orderResource;

    /**
     * @var PayshopSDKClientFactory
     */
    private PayshopSDKClientFactory $payshopSDKClientFactory;

    /**
     * @var TransactionSearchResultInterfaceFactory
     */
    private TransactionSearchResultInterfaceFactory $transactions;

    /**
     * ProcessWebhookEvent constructor.
     *
     * @param OrderResourceInterface $orderResource
     * @param PayshopSDKClientFactory $payshopSDKClientFactory
     * @param TransactionSearchResultInterfaceFactory $transactionSearchResultInterfaceFactory
     */
    public function __construct(
        OrderResourceInterface $orderResource,
        PayshopSDKClientFactory $payshopSDKClientFactory,
        TransactionSearchResultInterfaceFactory $transactionSearchResultInterfaceFactory
    ) {
        $this->orderResource = $orderResource;
        $this->payshopSDKClientFactory = $payshopSDKClientFactory;
        $this->transactions = $transactionSearchResultInterfaceFactory;
    }
    
    /**
     * Process webhook events
     *
     * @param array $eventBasicInformation
     * @return void
     */
    public function execute(array $eventBasicInformation): void
    {
        $eventReponse = $this->getApiEvent($eventBasicInformation['event']);
        $event = $eventReponse['response'];

        $order = $this->getOrderByInstrumentId($event['instrument']['id']);

        if ($event['type'] == 'payment.success') {
            $this->paymentSuccess($order, $event['payment']['id']);
            return;
        }

        if ($this->isWBWay($event) || $this->isDeclined($event)) {
            $this->mbWayDeclined($order);
            return;
        }
    }

    /**
     * Get the event from the API
     *
     * @param string $eventId
     * @return array
     */
    private function getApiEvent(string $eventId): array
    {
        $payshopClient = $this->payshopSDKClientFactory->getInstance();
        $eventReponse = $payshopClient->getEvent($eventId);

        $this->checkEventReponse($eventReponse);

        return $eventReponse;
    }

    /**
     * Get order by instrument id
     *
     * @param string $instrumentId
     * @return Order|null
     */
    private function getOrderByInstrumentId(string $instrumentId): ?Order
    {
        /** @var \Magento\Sales\Model\Order\Payment\Transaction $transactionFilter */
        $transactionFilter = $this->transactions->create();
        $searchCriteria = $transactionFilter->addFilter('txn_id', $instrumentId);
        $transactions = $searchCriteria->getItems();
        $transation = array_shift($transactions);

        return $transation->getOrder();
    }

    /**
     * Change order status to payment success
     *
     * @param Order $order
     * @param string $paymentId
     * @return void
     */
    private function paymentSuccess(Order $order, string $paymentId): void
    {
        $order->setState(Order::STATE_PROCESSING);
        $order->setStatus(Order::STATE_PROCESSING);
        $order->addStatusToHistory(Order::STATE_PROCESSING, __('Successfully registered payment'), true);
        $order->setTotalPaid($order->getGrandTotal());

        $order->getPayment()->setAdditionalInformation('payment_id', $paymentId);

        $this->orderResource->save($order);
    }

    /**
     * Check the event response
     *
     * @param array $response
     * @return void
     * @throws Exception
     */
    private function checkEventReponse(array $response): void
    {
        if ($response['status'] != '200') {
            throw new \Exception(__("Get event on payshop api error"));
        }
    }

    /**
     * Check if event is  WBWay
     *
     * @param array $event
     * @return bool
     */
    private function isWBWay(array $event): bool
    {
        return $event['charge']['charge_type'] == "mbway";
    }

    /**
     * Check if event is  declined
     *
     * @param array $event
     * @return bool
     */
    private function isDeclined(array $event): bool
    {
        return $event['instrument']['failure_code'] == "03.00.0000";
    }

    /**
     * Change order status to mbway declined
     *
     * @param Order $order
     * @return void
     */
    private function mbWayDeclined(Order $order): void
    {
        $order->setState(Order::STATE_CANCELED);
        $order->setStatus(Order::STATE_CANCELED);
        $order->addStatusToHistory(Order::STATE_CANCELED, __('Payment declined on MBWay'), true);

        $this->orderResource->save($order);
    }
}